#include <stdio.h>		/* for printf */
#include <stdlib.h>		/* for exit */
#include <getopt.h>
#include "grid_agents.h"


t_params parameters = { SMART_STEP, NEWTON,
  0.05, 0.05, 0.05, 0.05, 1.0, 1.0, 1.0, 1.0, 1.0, 0.5,
  200, -1, "obj", -1, "", ""
};

void print_param(t_params param)
{
  printf
      ("SCALING_HEURISTIC = %d\n"
       "NEWTON_HEURISTIC = %d\n"
       "GAMMA_RHO = %.20f\n"
       "GAMMA_NONRHO = %.20f\n"
       "GAMMA_LAMBDA = %.20f\n"
       "GAMMA_MU = %.20f\n"
       "RHO_INIT = %f\n"
       "NONRHO_INIT = %.20f\n"
       "LAMBDA_INIT = %f\n"
       "MU_INIT = %f\n"
       "NORM_FACTOR = %f\n"
       "DIV0_FACTOR = %f\n"
       "NB_ITERATIONS = %d\n"
       "INIT_TIME = %d\n"
       "input_file = %s\n"
       "output_file = %s\n",
       param.SCALING_HEURISTIC, param.NEWTON_HEURISTIC, param.GAMMA_RHO,
       param.GAMMA_NONRHO, param.GAMMA_LAMBDA, param.GAMMA_MU,
       param.RHO_INIT, param.NONRHO_INIT, param.LAMBDA_INIT, param.MU_INIT,
       param.NORM_FACTOR, param.DIV0_FACTOR, param.NB_ITERATIONS,
       param.INIT_TIME, param.input_file, param.output_file);
}


int main_parse(int argc, char **argv, char *platform, char *deployment)
{
  int opt = 0;
  int option_index = 0;
  static struct option long_options[] = {
    {"SCALING_HEURISTIC", 1, 0, 1},
    {"NEWTON_HEURISTIC", 1, 0, 2},
    {"GAMMA_RHO", 1, 0, 3},
    {"GAMMA_NONRHO", 1, 0, 4},
    {"GAMMA_LAMBDA", 1, 0, 5},
    {"GAMMA_MU", 1, 0, 6},
    {"RHO_INIT", 1, 0, 7},
    {"NONRHO_INIT", 1, 0, 8},
    {"LAMBDA_INIT", 1, 0, 9},
    {"MU_INIT", 1, 0, 10},
    {"NORM_FACTOR", 1, 0, 11},
    {"DIV0_FACTOR", 1, 0, 12},
    {"NB_ITERATIONS", 1, 0, 13},
    {"INIT_TIME", 1, 0, 14},
    {"platform", 1, 0, 'p'},
    {"deployment", 1, 0, 'd'},
    {"input", 1, 0, 'i'},
    {"output", 1, 0, 'o'},
    {0, 0, 0, 0}
  };


  while ((opt =
	  getopt_long(argc, argv, "i:p:d:o:", long_options,
		      &option_index)) != -1) {
    switch (opt) {
    case 'p':
      strcpy(platform, optarg);
      break;
    case 'd':
          strcpy(deployment, optarg);
          break;
    case 'i':
      sprintf(parameters.input_file, "%s", optarg);
      break;
    case 'o':
      sprintf(parameters.output_file, "%s", optarg);
      break;
    case 1:
      parameters.SCALING_HEURISTIC = atoi(optarg);
      break;
    case 2:
      parameters.NEWTON_HEURISTIC = atoi(optarg);
      break;
    case 3:
      parameters.GAMMA_RHO = atof(optarg);
      break;
    case 4:
      parameters.GAMMA_NONRHO = atof(optarg);
      break;
    case 5:
      parameters.GAMMA_LAMBDA = atof(optarg);
      break;
    case 6:
      parameters.GAMMA_MU = atof(optarg);
      break;
    case 7:
      parameters.RHO_INIT = atof(optarg);
      break;
    case 8:
      parameters.NONRHO_INIT = atof(optarg);
      break;
    case 9:
      parameters.LAMBDA_INIT = atof(optarg);
      break;
    case 10:
      parameters.MU_INIT = atof(optarg);
      break;
    case 11:
      parameters.NORM_FACTOR = atof(optarg);
      break;
    case 12:
      parameters.DIV0_FACTOR = atof(optarg);
      break;
    case 13:
      parameters.NB_ITERATIONS = atoi(optarg);
      break;
    case 14:
      parameters.INIT_TIME = atoi(optarg);
      break;

    default:
      printf("Unknown argument code : %d\n", opt);
      exit(1);
    }
  }
  //Test a posteriori, si certaines entrees sont incorectes.
  char synopsis[] = "Usage : grid_main -p <FILE> -d <FILE> [options]\n";
  char vide[] = "";
  if (!(strcmp(platform, vide) && strcmp(deployment, vide))) {
    printf("%s", synopsis);
    exit(-1);
  }
  FILE *fichier = NULL;
  fichier = fopen(platform, "r+");
  if (!fichier) {
    printf("Cannot read platform file %s\n", platform);
    exit(-1);
  }
  fclose(fichier);
  FILE *fichier2 = NULL;
  fichier2 = fopen(deployment, "r+");
  if (!fichier2) {
    printf("Cannot read deployment file %s\n", deployment);
    exit(-1);
  }
  fclose(fichier2);

  if (parameters.INIT_TIME >= 0) {
    FILE *fichier3 = NULL;
    fichier3 = fopen(parameters.input_file, "r+");
    if (!fichier3) {
      printf("Cannot read input file %s\nINIT_TIME is set to -1.\n",
	     parameters.input_file);
      parameters.INIT_TIME = -1;
    }
    fclose(fichier3);
  } else {
    parameters.INIT_TIME = -1;
  }

  print_param(parameters);
  return 0;
}
